import { __assign, __decorate, __metadata } from 'tslib';
import { Renderer2, ElementRef, Input, HostListener, Directive, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

// Copyright (c) 2019-2020 The Authors
/**
 * Default highlight styles: `none` for hover and `none` for focus.
 */
var DEFAULT_STYLE_CONFIG = {
    hover: 'none',
    focus: 'none',
};
/**
 * Default debounce time for mouse/focus events.
 */
var DEFAULT_DEBOUNCE_TIME = 50;
var HighlightDirective = /** @class */ (function () {
    function HighlightDirective(_renderer, _el) {
        this._renderer = _renderer;
        this._el = _el;
        this._style = __assign({}, DEFAULT_STYLE_CONFIG);
        this._debounceTime = DEFAULT_DEBOUNCE_TIME;
        // We are only interested in the event. The value of the subject is not used.
        this._hovered$ = new Subject();
        // This is the hover state that can be checked when the _hovered$ subject
        // emits
        this._isHovered = false;
        // We are only interested in the event. The value of the subject is not used.
        this._focused$ = new Subject();
        // This is the focus state that can be checked when the _hovered$ subject
        // emits
        this._isFocused = false;
        this._subscriptions = {
            hover: undefined,
            focus: undefined,
        };
    }
    Object.defineProperty(HighlightDirective.prototype, "highlightStyle", {
        /**
         * The currently set highlight styles for the directive.
         */
        get: function () {
            return this._style;
        },
        set: function (value) {
            this._updateStyleConfig(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighlightDirective.prototype, "hcHighlightDebounce", {
        /**
         * The currently set debounce time for the directive.
         *
         * Events are debounced to limit the frequency at which hover and/or focus
         * styles are applied/removed and provide a nicer user experience. A value of
         * `0` completely disables debouncing.
         */
        get: function () {
            return this._debounceTime;
        },
        set: function (value) {
            this._updateDebounceTime(value);
        },
        enumerable: true,
        configurable: true
    });
    HighlightDirective.prototype.ngOnDestroy = function () {
        this._stopFocusSubscription();
        this._stopHoverSubscription();
    };
    HighlightDirective.prototype.onMouseEnter = function () {
        this._isHovered = true;
        this._hovered$.next();
    };
    HighlightDirective.prototype.onMouseLeave = function () {
        this._isHovered = false;
        this._hovered$.next();
    };
    HighlightDirective.prototype.onFocusIn = function () {
        this._isFocused = true;
        this._focused$.next();
    };
    HighlightDirective.prototype.onFocusOut = function () {
        this._isFocused = false;
        this._focused$.next();
    };
    HighlightDirective.prototype._updateStyleConfig = function (value) {
        // Merge the provided config with the default config
        var newStyle = __assign({}, DEFAULT_STYLE_CONFIG);
        value &&
            (newStyle = __assign(__assign({}, newStyle), value));
        // Remove the old effects if the new ones are different,
        // they will be applied again with the new config as appropriate
        if (newStyle.hover != this._style.hover)
            this._removeEffect('hover');
        if (newStyle.focus != this._style.focus)
            this._removeEffect('focus');
        // Switch the style config to the new one and restart the subscriptions as
        // appropriate
        this._style = newStyle;
        // If the new style is 'none', stop the subscription
        if (newStyle.hover == 'none') {
            this._stopHoverSubscription();
            this._removeEffect('hover');
        }
        else {
            // Start the subscription if it has not been started already
            if (!this._subscriptions.hover) {
                this._startHoverSubscription();
            }
            this._hovered$.next();
        }
        if (newStyle.focus == 'none') {
            this._stopFocusSubscription();
            this._removeEffect('focus');
            // Restore the hover effect if the component is hovered
            if (this._isHovered) {
                this._applyEffect('hover');
            }
        }
        else {
            // Start the subscription if it has not been started already
            if (!this._subscriptions.focus) {
                this._startFocusSubscription();
            }
            this._focused$.next();
        }
    };
    HighlightDirective.prototype._updateDebounceTime = function (value) {
        // Adjust debounceTime if it is negative
        var adjustedDebounceTime = DEFAULT_DEBOUNCE_TIME;
        if (value) {
            adjustedDebounceTime = value > 0 ? value : 0;
        }
        if (this._debounceTime != adjustedDebounceTime) {
            this._debounceTime = adjustedDebounceTime;
            if (this._subscriptions.hover) {
                this._stopHoverSubscription();
                this._startHoverSubscription();
            }
            if (this._subscriptions.focus) {
                this._stopFocusSubscription();
                this._startFocusSubscription();
            }
        }
    };
    HighlightDirective.prototype._startHoverSubscription = function () {
        var _this = this;
        this._subscriptions.hover = this._hovered$
            .pipe(debounceTime(this._debounceTime))
            .subscribe(function () {
            // Do not manipulate the hover effect if the host is focused and the
            // focus style is not `none`
            if (!_this._isFocused || _this._style.focus == 'none') {
                if (_this._isHovered) {
                    _this._applyEffect('hover');
                }
                else {
                    _this._removeEffect('hover');
                }
            }
        });
    };
    HighlightDirective.prototype._stopHoverSubscription = function () {
        var _a;
        (_a = this._subscriptions.hover) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.hover = undefined;
    };
    HighlightDirective.prototype._startFocusSubscription = function () {
        var _this = this;
        this._subscriptions.focus = this._focused$
            .pipe(debounceTime(this._debounceTime))
            .subscribe(function () {
            if (_this._isFocused) {
                _this._applyEffect('focus');
                // If the host is hovered, then remove the hover effect
                if (_this._isHovered && _this._style.focus != 'none')
                    _this._removeEffect('hover');
            }
            else {
                _this._removeEffect('focus');
                // If the host is hovered, then apply the hover effect
                if (_this._isHovered)
                    _this._applyEffect('hover');
            }
        });
    };
    HighlightDirective.prototype._stopFocusSubscription = function () {
        var _a;
        (_a = this._subscriptions.focus) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.focus = undefined;
    };
    HighlightDirective.prototype._applyEffect = function (effect) {
        var style = this._style[effect];
        if (style != 'none') {
            this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-" + style);
        }
    };
    HighlightDirective.prototype._removeEffect = function (effect) {
        var style = this._style[effect];
        if (style != 'none') {
            this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-" + style);
        }
    };
    HighlightDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        Input('hcHighlight'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], HighlightDirective.prototype, "highlightStyle", null);
    __decorate([
        Input(),
        __metadata("design:type", Number),
        __metadata("design:paramtypes", [Number])
    ], HighlightDirective.prototype, "hcHighlightDebounce", null);
    __decorate([
        HostListener('mouseenter'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseEnter", null);
    __decorate([
        HostListener('mouseleave'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseLeave", null);
    __decorate([
        HostListener('focusin'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusIn", null);
    __decorate([
        HostListener('focusout'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusOut", null);
    HighlightDirective = __decorate([
        Directive({
            selector: '[hcHighlight]',
        }),
        __metadata("design:paramtypes", [Renderer2, ElementRef])
    ], HighlightDirective);
    return HighlightDirective;
}());

// Copyright (c) 2019-2020 The Authors
var HighlightModule = /** @class */ (function () {
    function HighlightModule() {
    }
    HighlightModule = __decorate([
        NgModule({
            declarations: [HighlightDirective],
            imports: [CommonModule],
            exports: [HighlightDirective],
        })
    ], HighlightModule);
    return HighlightModule;
}());

// Copyright (c) 2019-2020 The Authors

// Copyright (c) 2019-2020 The Authors

/**
 * Generated bundle index. Do not edit.
 */

export { DEFAULT_DEBOUNCE_TIME, DEFAULT_STYLE_CONFIG, HighlightDirective, HighlightModule };
//# sourceMappingURL=npcz-ngx-highlight.js.map
